@@ -1,5 +1,6 @@
 import asyncio
 import contextlib
+from datetime import datetime
 import logging
 import time
 from typing import Callable, Coroutine, Dict, List, Optional, Tuple
@@ -29,12 +30,14 @@ from .const import (
 )
 from .protocol import (
     POWER_RESTORE_BYTES_TO_POWER_RESTORE,
+    REMOTE_CONFIG_BYTES_TO_REMOTE_CONFIG,
     PowerRestoreState,
     PowerRestoreStates,
     ProtocolLEDENET8Byte,
     ProtocolLEDENETAddressableA3,
     ProtocolLEDENETAddressableChristmas,
     ProtocolLEDENETOriginal,
+    RemoteConfig,
 )
 from .scanner import FluxLEDDiscovery
 from .utils import color_temp_to_white_levels, rgbw_brightness, rgbww_brightness
@@ -77,9 +80,12 @@ class AIOWifiLedBulb(LEDENETDevice):
         super().__init__(ipaddr, port, timeout, discovery)
         self._lock = asyncio.Lock()
         self._aio_protocol: Optional[AIOLEDENETProtocol] = None
+        self._get_time_lock: asyncio.Lock = asyncio.Lock()
+        self._get_time_future: Optional[asyncio.Future[bool]] = None
         self._power_restore_future: "asyncio.Future[bool]" = asyncio.Future()
         self._device_config_lock: asyncio.Lock = asyncio.Lock()
         self._device_config_future: asyncio.Future[bool] = asyncio.Future()
+        self._remote_config_future: asyncio.Future[bool] = asyncio.Future()
         self._device_config_setup = False
         self._on_futures: List["asyncio.Future[bool]"] = []
         self._off_futures: List["asyncio.Future[bool]"] = []
@@ -103,10 +109,11 @@ class AIOWifiLedBulb(LEDENETDevice):
         assert self._protocol is not None
         if isinstance(self._protocol, ALL_IC_PROTOCOLS):
             await self._async_device_config_setup()
-            return
+        hardware = self.hardware
+        if hardware is not None and hardware.remote_24g_controls:
+            await self._async_remote_config_setup()
         if self.device_type == DeviceType.Switch:
             await self._async_switch_setup()
-            return
         _LOGGER.debug(
             "%s: device_config: wiring=%s operating_mode=%s",
             self.ipaddr,
@@ -114,6 +121,15 @@ class AIOWifiLedBulb(LEDENETDevice):
             self.operating_mode,
         )
 
+    async def _async_remote_config_setup(self) -> None:
+        """Setup remote config."""
+        assert self._protocol is not None
+        await self._async_send_msg(self._protocol.construct_query_remote_config())
+        try:
+            await asyncio.wait_for(self._remote_config_future, timeout=self.timeout)
+        except asyncio.TimeoutError:
+            _LOGGER.warning("%s: Could not determine 2.4ghz remote config", self.ipaddr)
+
     async def _async_switch_setup(self) -> None:
         """Setup a switch."""
         assert self._protocol is not None
@@ -194,6 +210,7 @@ class AIOWifiLedBulb(LEDENETDevice):
     async def async_turn_on(self) -> bool:
         """Turn on the device."""
         calls = (
+            self._async_turn_on,
             self._async_turn_on,
             self._async_turn_off_on,
             self._async_turn_on,
@@ -221,6 +238,7 @@ class AIOWifiLedBulb(LEDENETDevice):
     async def async_turn_off(self) -> bool:
         """Turn off the device."""
         calls = (
+            self._async_turn_off,
             self._async_turn_off,
             self._async_turn_on_off,
             self._async_turn_off,
@@ -306,11 +324,12 @@ class AIOWifiLedBulb(LEDENETDevice):
         )
 
     async def _async_process_levels_change(
-        self, msg: bytearray, updates: Dict[str, int]
+        self, msgs: List[bytearray], updates: Dict[str, int]
     ) -> None:
         """Process and send a levels change."""
         self._set_transition_complete_time()
-        await self._async_send_msg(msg)
+        for msg in msgs:
+            await self._async_send_msg(msg)
         if updates:
             self._replace_raw_state(updates)
 
@@ -509,6 +528,42 @@ class AIOWifiLedBulb(LEDENETDevice):
             if isinstance(self._protocol, ALL_IC_PROTOCOLS):
                 await self._async_device_config_resync()
 
+    async def async_unpair_remotes(self) -> None:
+        """Unpair 2.4ghz remotes."""
+        assert self._protocol is not None
+        if self.paired_remotes is None:
+            raise ValueError("{self.model} does support unpairing remotes")
+        await self._async_send_msg(self._protocol.construct_unpair_remotes())
+        await self._async_send_msg(self._protocol.construct_query_remote_config())
+
+    async def async_config_remotes(self, remote_config: RemoteConfig) -> None:
+        """Change remote config."""
+        assert self._protocol is not None
+        if self.paired_remotes is None:
+            raise ValueError("{self.model} does support unpairing remotes")
+        await self._async_send_msg(
+            self._protocol.construct_remote_config(remote_config)
+        )
+        await self._async_send_msg(self._protocol.construct_query_remote_config())
+
+    async def async_get_time(self) -> Optional[datetime]:
+        """Get the current time."""
+        assert self._protocol is not None
+        await self._async_send_msg(self._protocol.construct_get_time())
+        async with self._get_time_lock:
+            self._get_time_future = asyncio.Future()
+            try:
+                await asyncio.wait_for(self._get_time_future, timeout=self.timeout)
+            except asyncio.TimeoutError:
+                _LOGGER.warning("%s: Could not get time from the device", self.ipaddr)
+                return None
+            return self._last_time
+
+    async def async_set_time(self, time: Optional[datetime] = None) -> None:
+        """Set the current time."""
+        assert self._protocol is not None
+        await self._async_send_msg(self._protocol.construct_set_time(time))
+
     async def _async_device_config_resync(self) -> None:
         await asyncio.sleep(DEVICE_CONFIG_WAIT_SECONDS)
         await self._async_device_config_setup()
@@ -579,11 +634,16 @@ class AIOWifiLedBulb(LEDENETDevice):
             self._async_process_state_response(msg)
         elif self._protocol.is_valid_power_state_response(msg):
             self.process_power_state_response(msg)
+        elif self._protocol.is_valid_get_time_response(msg):
+            self.process_time_response(msg)
         elif self._protocol.is_valid_device_config_response(msg):
             self.process_device_config_response(msg)
             changed_state = True
         elif self._protocol.is_valid_power_restore_state_response(msg):
             self.process_power_restore_state_response(msg)
+        elif self._protocol.is_valid_remote_config_response(msg):
+            self.process_remote_config_response(msg)
+            changed_state = True
         else:
             return
         if not changed_state and self.raw_state == prev_state:
@@ -627,6 +687,28 @@ class AIOWifiLedBulb(LEDENETDevice):
         if not self._device_config_future.done():
             self._device_config_future.set_result(True)
 
+    def process_time_response(self, msg: bytes) -> None:
+        """Process an time response."""
+        assert self._protocol is not None
+        self._last_time = self._protocol.parse_get_time(msg)
+        if self._get_time_future and not self._get_time_future.done():
+            self._get_time_future.set_result(True)
+
+    def process_remote_config_response(self, msg: bytes) -> None:
+        """Process a 2.4ghz remote config response."""
+        # 2b 03 00 02 00 00 00 00 00 00 00 00 00 30
+        #  0  1  2  3
+        self._paired_remotes = msg[3]
+        self._remote_config = REMOTE_CONFIG_BYTES_TO_REMOTE_CONFIG.get(msg[1])
+        _LOGGER.debug(
+            "%s: remote_config: config=%s paired_remotes=%s",
+            self.ipaddr,
+            self._remote_config,
+            self._paired_remotes,
+        )
+        if not self._remote_config_future.done():
+            self._remote_config_future.set_result(True)
+
     async def _async_send_msg(self, msg: bytearray) -> None:
         """Write a message on the socket."""
         if not self._aio_protocol:
