@@ -7,6 +7,7 @@ from typing import Dict, Iterable, List, Optional, Set, Tuple, Type, Union
 
 from .const import (  # imported for back compat, remove once Home Assistant no longer uses
     ADDRESSABLE_STATE_CHANGE_LATENCY,
+    ATTR_MODEL,
     ATTR_MODEL_DESCRIPTION,
     CHANNEL_STATES,
     COLOR_MODE_CCT,
@@ -42,7 +43,14 @@ from .const import (  # imported for back compat, remove once Home Assistant no
     WRITE_ALL_WHITES,
     LevelWriteMode,
 )
-from .models_db import BASE_MODE_MAP, LEDENETModel, get_model, is_known_model
+from .models_db import (
+    BASE_MODE_MAP,
+    HARDWARE_MAP,
+    LEDENETHardware,
+    LEDENETModel,
+    get_model,
+    is_known_model,
+)
 from .pattern import (
     ADDRESSABLE_EFFECT_ID_NAME,
     ADDRESSABLE_EFFECT_NAME_ID,
@@ -90,6 +98,7 @@ from .protocol import (
     ProtocolLEDENETCCT,
     ProtocolLEDENETOriginal,
     ProtocolLEDENETOriginalCCT,
+    RemoteConfig,
 )
 from .scanner import FluxLEDDiscovery, is_legacy_device
 from .timer import BuiltInTimer
@@ -190,6 +199,8 @@ class LEDENETDevice:
         self.available: Optional[bool] = None
         self._model_num: Optional[int] = None
         self._model_data: Optional[LEDENETModel] = None
+        self._paired_remotes: Optional[int] = None
+        self._remote_config: Optional[RemoteConfig] = None
         self._discovery = discovery
         self._protocol: Optional[PROTOCOL_TYPES] = None
         self._mode: Optional[str] = None
@@ -231,6 +242,27 @@ class LEDENETDevice:
         """Set the discovery data."""
         self._discovery = value
 
+    @property
+    def hardware(self) -> Optional[LEDENETHardware]:
+        """Retrurn the hardware mapping for the device."""
+        if not self._discovery or ATTR_MODEL not in self._discovery:
+            return None
+        assert self._discovery is not None
+        assert self._discovery[ATTR_MODEL] is not None
+        model = self._discovery[ATTR_MODEL]
+        assert isinstance(model, str)
+        return HARDWARE_MAP.get(model)
+
+    @property
+    def paired_remotes(self) -> Optional[int]:
+        """Return the number of paired remotes or None if not supported."""
+        return self._paired_remotes
+
+    @property
+    def remote_config(self) -> Optional[RemoteConfig]:
+        """Return the number of remote config or None if not supported."""
+        return self._remote_config
+
     @property
     def speed_adjust_off(self) -> int:
         """Return true if the speed of an effect can be adjusted while off."""
@@ -344,6 +376,12 @@ class LEDENETDevice:
     def _internal_color_modes(self) -> Set[str]:
         """The internal available color modes."""
         assert self.raw_state is not None
+        if (
+            self._device_config is not None
+            # Currently this is only the SK6812RGBW strips on 0xA3
+            and self._device_config.operating_mode == COLOR_MODE_RGBW
+        ):
+            return {COLOR_MODE_RGBW}
         if not is_known_model(self.model_num):
             # Default mode is RGB
             return BASE_MODE_MAP.get(self.raw_state.mode & 0x0F, {DEFAULT_MODE})
@@ -921,7 +959,7 @@ class LEDENETDevice:
     def getSpeed(self) -> int:
         return self.speed
 
-    def _generate_random_levels_change(self) -> Tuple[bytearray, Dict[str, int]]:
+    def _generate_random_levels_change(self) -> Tuple[List[bytearray], Dict[str, int]]:
         """Generate a random levels change."""
         channels = {STATE_WARM_WHITE}
         if COLOR_MODES_RGB.intersection(self.color_modes):
@@ -940,7 +978,7 @@ class LEDENETDevice:
         channels: Dict[str, Optional[int]],
         persist: bool = True,
         brightness: Optional[int] = None,
-    ) -> Tuple[bytearray, Dict[str, int]]:
+    ) -> Tuple[List[bytearray], Dict[str, int]]:
         """Generate the levels change request."""
         channel_map = self.model_data.channel_map
         if channel_map:
@@ -963,16 +1001,16 @@ class LEDENETDevice:
         if brightness is not None and r is not None and g is not None and b is not None:
             (r, g, b) = self._calculateBrightness((r, g, b), brightness)
 
-        r_value = 0 if r is None else int(r)
-        g_value = 0 if g is None else int(g)
-        b_value = 0 if b is None else int(b)
-        w_value = 0 if w is None else int(w)
+        r_value = None if r is None else int(r)
+        g_value = None if g is None else int(g)
+        b_value = None if b is None else int(b)
+        w_value = None if w is None else int(w)
         # ProtocolLEDENET9Byte devices support two white outputs for cold and warm.
         if w2 is None:
             # If we're only setting a single white value,
             # we set the second output to be the same as the first
             w2_value = (
-                int(w) if w is not None and self.color_mode != COLOR_MODE_CCT else 0
+                int(w) if w is not None and self.color_mode != COLOR_MODE_CCT else None
             )
         else:
             w2_value = int(w2)
@@ -985,36 +1023,19 @@ class LEDENETDevice:
             elif r is None and g is None and b is None:
                 write_mode = LevelWriteMode.WHITES
 
-        _LOGGER.debug(
-            "%s: _generate_levels_change using %s: persist=%s r=%s/%s, g=%s/%s b=%s/%s, w=%s/%s w2=%s/%s write_mode=%s/%s",
-            self.ipaddr,
-            self.protocol,
-            persist,
-            r_value,
-            f"0x{r_value:02X}",
-            g_value,
-            f"0x{g_value:02X}",
-            b_value,
-            f"0x{b_value:02X}",
-            w_value,
-            f"0x{w_value:02X}",
-            w2_value,
-            f"0x{w2_value:02X}",
-            write_mode,
-            f"0x{write_mode.value:02X}",
-        )
-
         assert self._protocol is not None
-        msg = self._protocol.construct_levels_change(
+        msgs = self._protocol.construct_levels_change(
             persist, r_value, g_value, b_value, w_value, w2_value, write_mode
         )
         updates = {}
         multi_mode = self.multi_color_mode
         if multi_mode or write_mode in WRITE_ALL_COLORS:
-            updates.update({"red": r_value, "green": g_value, "blue": b_value})
+            updates.update(
+                {"red": r_value or 0, "green": g_value or 0, "blue": b_value or 0}
+            )
         if multi_mode or write_mode in WRITE_ALL_WHITES:
-            updates.update({"warm_white": w_value, "cool_white": w2_value})
-        return msg, updates
+            updates.update({"warm_white": w_value or 0, "cool_white": w2_value or 0})
+        return msgs, updates
 
     def _set_transition_complete_time(self) -> None:
         """Set the time we expect the transition will be completed.
