@@ -56,7 +56,7 @@ from .protocol import (
 
 # BL likely means BL602 chips
 MODEL_INFO_NAMES = {
-    "ZG-LX-FL": "Flood",  # Seen on 24w Flood light
+    "ZG-LX-FL": "",  # Seen on 24w Flood light
     "ZG-BL": "",  # unknown
     "CL-BL": "",  # Send on the 0x1C table lamp
     "ZG-BL-IR": "IR",
@@ -73,7 +73,7 @@ MODEL_INFO_NAMES = {
     "ZG-BL-5V": "5v",
     "ZG-LX": "",  # Seen on floor lamp, v2 addressable, and Single channel controller
     "ZG-LX-UART": "",  # Seen on UK xmas lights 0x33, fairy controller, and lytworx
-    "ZG-BL-PWM": "Flood",  # Seen on 40w Flood Light
+    "ZG-BL-PWM": "",  # Seen on 40w Flood Light
     "ZG-ZW2": "",  # seen on 0x97 socket
     "ZGIR44": "44 Key IR",
     "IR_ZG": "IR",
@@ -97,7 +97,9 @@ class LEDENETChip(Enum):
 class LEDENETHardware:
     model: str  # The model string
     chip: LEDENETChip
-    rf_remote: bool
+    remote_rf: bool  # legacy rf remote
+    remote_24g: bool  # 2.4ghz remote
+    remote_24g_controls: bool  # 2.4ghz remote controls (pair/unpair remotes)
 
 
 BASE_MODE_MAP = {
@@ -343,134 +345,187 @@ HARDWARE = [
     LEDENETHardware(
         model="AK001-ZJ100",
         chip=LEDENETChip.ESP8266,
-        rf_remote=False,  # verified
+        remote_rf=False,  # verified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ200",
         chip=LEDENETChip.ESP8266,
-        rf_remote=False,  # verified
+        remote_rf=False,  # verified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ210",  # might also be "AK001-ZJ2100"
         chip=LEDENETChip.ESP8266,  # verified
-        rf_remote=False,  # verified
+        remote_rf=False,  # verified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2101",
         chip=LEDENETChip.ESP8266,
-        rf_remote=False,
+        remote_rf=False,
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2104",
         chip=LEDENETChip.ESP8266,
-        rf_remote=False,  # verified
+        remote_rf=False,  # verified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2134",  # seen in smart plugs only?
         chip=LEDENETChip.S82GESNC,  # couldn't get the device appart
-        rf_remote=False,
+        remote_rf=False,
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2145",
         chip=LEDENETChip.BL602,
-        rf_remote=False,  # verified
+        remote_rf=False,  # verified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2146",  # seen in smart plugs & Controller RGBCW, but RF remote isn't supported on plugs
         chip=LEDENETChip.BL602,  # verified
-        rf_remote=True,  # verified
+        remote_rf=False,  # verified
+        remote_24g=True,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2147",  # seen on Controller RGBW
         chip=LEDENETChip.BL602,
-        rf_remote=True,  # verified
+        remote_rf=False,  # verified
+        remote_24g=True,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="AK001-ZJ2148",  # seen on older Addressable v3
         chip=LEDENETChip.BL602,
-        rf_remote=True,  # verified
+        remote_rf=True,  # verified
+        remote_24g=True,
+        remote_24g_controls=True,
     ),
     LEDENETHardware(
         model="AK001-ZJ2149",  # seen on newer Addressable v3
         chip=LEDENETChip.BL602,
-        rf_remote=True,  # verified
+        remote_rf=True,  # verified
+        remote_24g=True,
+        remote_24g_controls=True,
     ),
     LEDENETHardware(
         model="HF-A11",  # reported older large box controllers (may be original proto)
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-A11-ZJ002",  # reported older large box controllers (may be original proto)
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-A11-ZJ004",  # reported older large box controllers (may be original proto)
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-A11-ZJ2",  # reported older large box controllers (may be original proto)
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-A11-ZJ201",  # reported older large box controllers (may be original proto)
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-ZJ2001",  # reported older large box controllers (may be original proto)
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100",  # reported on older UFO
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-",
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-0",  # reported on older UFO
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-1",  # reported on older UFO
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-ZJ001",
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-ZJ011",
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-ZJ002",  # seen on older UFO
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
     LEDENETHardware(
         model="HF-LPB100-ZJ200",  # seen on RGBW Downlight
         chip=LEDENETChip.HFLPB100,
-        rf_remote=False,  # unverified
+        remote_rf=False,  # unverified
+        remote_24g=False,
+        remote_24g_controls=False,
     ),
 ]
 
+HARDWARE_MAP: Dict[str, LEDENETHardware] = {model.model: model for model in HARDWARE}
+
+
 MODELS = [
     LEDENETModel(
         model_num=0x01,
-        models=[],
+        models=["HF-A11-ZJ002"],
         description="Legacy Controller RGB",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_ORIGINAL)],
@@ -511,6 +566,9 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x04,
+        # AK001-ZJ200 == v5.20
+        # There are a limited set of these devices that are the mini version
+        # but most are UFOs
         models=[
             "HF-LPB100",
             "HF-LPB100-0",
@@ -530,8 +588,8 @@ MODELS = [
     LEDENETModel(
         model_num=0x06,
         # "AK001-ZJ2104" == v1 has RF remote support
-        # "AK001-ZJ2145" == v2 has IR remote support
-        # "AK001-ZJ2147" == v3 has RF remote support
+        # "AK001-ZJ2145" == v2.03, v2.09 has IR remote support
+        # "AK001-ZJ2147" == v3.31 has 2.4ghz remote support
         models=[
             "AK001-ZJ100",
             "AK001-ZJ200",
@@ -616,15 +674,17 @@ MODELS = [
     LEDENETModel(
         # 'AK001-ZJ2104' likely supports turning on by effect/levels set
         # 'AK001-ZJ2104' is v7
-        model_num=0x0E,
-        models=["AK001-ZJ2104"],
+        # 'AK001-ZJ2148' is v9.75 with Remote and 2.4G remote settings
+        model_num=0x0E,  # Should be the same as 0x35
+        models=["AK001-ZJ2104", "AK001-ZJ2148"],
         description="Floor Lamp RGBCW",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
         protocols=[
+            MinVersionProtocol(9, PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS),
             MinVersionProtocol(7, PROTOCOL_LEDENET_9BYTE_AUTO_ON),
             MinVersionProtocol(0, PROTOCOL_LEDENET_9BYTE),
         ],
-        mode_to_color_mode={0x01: COLOR_MODES_RGB_CCT},
+        mode_to_color_mode={0x01: COLOR_MODES_RGB_CCT, 0x17: COLOR_MODES_RGB_CCT},
         color_modes=COLOR_MODES_RGB_CCT,
         channel_map={},
         microphone=False,
@@ -695,18 +755,6 @@ MODELS = [
         microphone=False,
         device_config=IMMUTABLE_DEVICE_CONFIG,
     ),
-    LEDENETModel(
-        model_num=0x1B,
-        models=[],
-        description="Spray Light",
-        always_writes_white_and_colors=False,  # Formerly rgbwprotocol
-        protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
-        mode_to_color_mode={},
-        color_modes={COLOR_MODE_RGB},
-        channel_map={},
-        microphone=False,
-        device_config=IMMUTABLE_DEVICE_CONFIG,
-    ),
     LEDENETModel(
         model_num=0x19,
         models=[],
@@ -719,9 +767,21 @@ MODELS = [
         microphone=False,
         device_config=IMMUTABLE_DEVICE_CONFIG,
     ),
+    LEDENETModel(
+        model_num=0x1B,
+        models=[],
+        description="Spray Light",
+        always_writes_white_and_colors=False,  # Formerly rgbwprotocol
+        protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
+        mode_to_color_mode={},
+        color_modes={COLOR_MODE_RGB},
+        channel_map={},
+        microphone=False,
+        device_config=IMMUTABLE_DEVICE_CONFIG,
+    ),
     LEDENETModel(
         model_num=0x1C,
-        # AK001-ZJ2149 has RF remote support
+        # AK001-ZJ2149 has RF remote support and can change 2.4G remote settings
         models=["AK001-ZJ2149"],
         description="Table Light CCT",
         always_writes_white_and_colors=False,  # Formerly rgbwprotocol
@@ -732,6 +792,22 @@ MODELS = [
         microphone=False,
         device_config=IMMUTABLE_DEVICE_CONFIG,
     ),
+    LEDENETModel(
+        model_num=0x1E,  # Should be the same as 0x35
+        models=[],
+        description="Ceiling Light RGBCW",
+        always_writes_white_and_colors=False,  # Formerly rgbwprotocol
+        protocols=[
+            MinVersionProtocol(9, PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS),
+            MinVersionProtocol(7, PROTOCOL_LEDENET_9BYTE_AUTO_ON),
+            MinVersionProtocol(0, PROTOCOL_LEDENET_9BYTE),
+        ],
+        mode_to_color_mode={0x01: COLOR_MODES_RGB_CCT, 0x17: COLOR_MODES_RGB_CCT},
+        color_modes=COLOR_MODES_RGB_CCT,
+        channel_map={},
+        microphone=False,
+        device_config=IMMUTABLE_DEVICE_CONFIG,
+    ),
     LEDENETModel(
         model_num=0x21,
         # 'AK001-ZJ200' is v1 but with new firmware it will change to v2
@@ -767,10 +843,11 @@ MODELS = [
         model_num=0x33,
         # 'AK001-ZJ100' == v3 - WIFI370 version
         # 'AK001-ZJ2104' == v7 supports turning on by effect/levels set
-        # 'AK001-ZJ2101' == v8 - no dimmable effects confirmed, confirmed auto on
+        # 'AK001-ZJ2101' == v8.61 - no dimmable effects confirmed, confirmed auto on
         # "AK001-ZJ2145" == v9 # no rf support!
-        # "AK001-ZJ2146" == v10 # rf support
-        # "AK001-ZJ2148" == v10.63 # rf support
+        # "AK001-ZJ2145" == v9.27 # w/IR
+        # "AK001-ZJ2146" == v10.48 # 2.4ghz support
+        # "AK001-ZJ2148" == v10.63 # 2.4ghz support, confirmed to be able to change 2.4G remote settings
         models=[
             "AK001-ZJ210",
             "AK001-ZJ2104",
@@ -797,7 +874,7 @@ MODELS = [
         # 'AK001-ZJ2101' and 'AK001-ZJ2104' is v7
         # 'AK001-ZJ2145' is v8
         # 'AK001-ZJ2146' is v9
-        # 'AK001-ZJ2147' is v9.7
+        # 'AK001-ZJ2147' is v9.7 (with RF remote control support)
         # 'AK001-ZJ2146' 40w flood light, newer smart bulbs (with RF remote control support)
         models=[
             "AK001-ZJ2101",
@@ -864,9 +941,9 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0x52,
-        models=[],
+        models=["AK001-ZJ200"],
         description="Bulb CCT",
-        always_writes_white_and_colors=False,  # Formerly rgbwprotocol
+        always_writes_white_and_colors=True,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
         color_modes={COLOR_MODE_CCT},  # Formerly rgbwcapable
@@ -895,7 +972,7 @@ MODELS = [
         model_num=0x62,
         models=[],
         description="Controller CCT",
-        always_writes_white_and_colors=False,  # Formerly rgbwprotocol
+        always_writes_white_and_colors=True,  # Formerly rgbwprotocol
         protocols=[MinVersionProtocol(0, PROTOCOL_LEDENET_8BYTE)],
         mode_to_color_mode={},
         color_modes={COLOR_MODE_CCT},  # Formerly rgbwcapable
@@ -984,6 +1061,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0xA1,
+        # AK001-ZJ210 = v3.18
         models=["AK001-ZJ210"],
         description="Addressable v1",
         always_writes_white_and_colors=False,
@@ -1010,7 +1088,7 @@ MODELS = [
     LEDENETModel(
         model_num=0xA3,
         # AK001-ZJ2147 has RF remote support
-        # AK001-ZJ2148 has RF remote support
+        # AK001-ZJ2148 has RF remote support, confirmed to be able to change 2.4G remote settings
         models=["AK001-ZJ2147", "AK001-ZJ2148"],
         description="Addressable v3",
         always_writes_white_and_colors=False,
@@ -1023,7 +1101,7 @@ MODELS = [
     ),
     LEDENETModel(
         model_num=0xA4,
-        models=["AK001-ZJ2149"],
+        models=[],
         description="Addressable v4",
         always_writes_white_and_colors=False,
         protocols=[
@@ -1032,7 +1110,21 @@ MODELS = [
         mode_to_color_mode={},
         color_modes=COLOR_MODES_ADDRESSABLE,
         channel_map={},
-        microphone=True,
+        microphone=True,  # may be false
+        device_config=NEW_ADDRESABLE_DEVICE_CONFIG,
+    ),
+    LEDENETModel(
+        model_num=0xA6,
+        models=[],
+        description="Addressable v6",
+        always_writes_white_and_colors=False,
+        protocols=[
+            MinVersionProtocol(0, PROTOCOL_LEDENET_ADDRESSABLE_A3)
+        ],  # Currently no difference from v3 proto
+        mode_to_color_mode={},
+        color_modes=COLOR_MODES_ADDRESSABLE,
+        channel_map={},
+        microphone=True,  # may be false
         device_config=NEW_ADDRESABLE_DEVICE_CONFIG,
     ),
     LEDENETModel(
