@@ -2,7 +2,9 @@
 
 from abc import abstractmethod
 import colorsys
+import contextlib
 from dataclasses import dataclass
+import datetime
 from enum import Enum
 import logging
 from typing import List, NamedTuple, Optional, Tuple, Union
@@ -25,6 +27,12 @@ from .const import (
 from .utils import utils, white_levels_to_scaled_color_temp
 
 
+class RemoteConfig(Enum):
+    DISABLED = 0x01
+    OPEN = 0x02
+    PAIRED_ONLY = 0x03
+
+
 class PowerRestoreState(Enum):
     ALWAYS_OFF = 0xFF
     ALWAYS_ON = 0x0F
@@ -60,6 +68,7 @@ class PowerRestoreStates:
 
 _LOGGER = logging.getLogger(__name__)
 
+
 # Protocol names
 PROTOCOL_LEDENET_ORIGINAL = "LEDENET_ORIGINAL"
 PROTOCOL_LEDENET_ORIGINAL_CCT = "LEDENET_ORIGINAL_CCT"
@@ -90,6 +99,8 @@ LEDENET_POWER_RESPONSE_LEN = 4
 LEDENET_ADDRESSABLE_STATE_RESPONSE_LEN = 25
 LEDENET_A1_DEVICE_CONFIG_RESPONSE_LEN = 12
 LEDENET_DEVICE_CONFIG_RESPONSE_LEN = 11
+LEDENET_REMOTE_CONFIG_RESPONSE_LEN = 14  # 2b 03 00 00 00 00 29 00 00 00 00 00 00 57
+LEDENET_REMOTE_CONFIG_TIME_RESPONSE_LEN = 12  # 10 14 16 01 02 10 26 20 07 00 0f a9
 
 MSG_ORIGINAL_POWER_STATE = "original_power_state"
 MSG_ORIGINAL_STATE = "original_state"
@@ -98,16 +109,20 @@ MSG_POWER_RESTORE_STATE = "power_restore_state"
 MSG_POWER_STATE = "power_state"
 MSG_STATE = "state"
 
+MSG_TIME = "time"
 MSG_MUSIC_MODE_STATE = "music_mode_state"
 MSG_ADDRESSABLE_STATE = "addressable_state"
 
 MSG_DEVICE_CONFIG = "device_config"
 MSG_A1_DEVICE_CONFIG = "a1_device_config"
-
+MSG_REMOTE_CONFIG = "remote_config"
 
 OUTER_MESSAGE_FIRST_BYTE = 0xB0
 
 MSG_UNIQUE_START = {
+    (0xF0, 0x11): MSG_TIME,
+    (0x0F, 0x11): MSG_TIME,
+    (0x00, 0x11): MSG_TIME,
     (0xF0, 0x71): MSG_POWER_STATE,
     (0x0F, 0x71): MSG_POWER_STATE,
     (0x00, 0x71): MSG_POWER_STATE,
@@ -122,9 +137,12 @@ MSG_UNIQUE_START = {
     (0x0F, 0x63): MSG_DEVICE_CONFIG,
     (0x63,): MSG_A1_DEVICE_CONFIG,
     (0x72,): MSG_MUSIC_MODE_STATE,
+    (0x2B,): MSG_REMOTE_CONFIG,
 }
 
 MSG_LENGTHS = {
+    MSG_TIME: LEDENET_REMOTE_CONFIG_TIME_RESPONSE_LEN,
+    MSG_REMOTE_CONFIG: LEDENET_REMOTE_CONFIG_RESPONSE_LEN,
     MSG_MUSIC_MODE_STATE: LEDNET_MUSIC_MODE_RESPONSE_LEN,
     MSG_POWER_STATE: LEDENET_POWER_RESPONSE_LEN,
     MSG_POWER_RESTORE_STATE: LEDENET_POWER_RESTORE_RESPONSE_LEN,
@@ -146,12 +164,17 @@ POWER_RESTORE_BYTES_TO_POWER_RESTORE = {
 }
 
 
+REMOTE_CONFIG_BYTES_TO_REMOTE_CONFIG = {
+    remote_config.value: remote_config for remote_config in RemoteConfig
+}
+
+
 def _message_type_from_start_of_msg(data: bytes) -> Optional[str]:
     if len(data) > 1:
         return MSG_UNIQUE_START.get(
             (data[0], data[1]), MSG_UNIQUE_START.get((data[0],))
         )
-    return MSG_UNIQUE_START.get((data[0],))
+    return MSG_UNIQUE_START.get((data[0],)) if len(data) else None
 
 
 class LEDENETOriginalRawState(NamedTuple):
@@ -457,6 +480,48 @@ class ProtocolBase:
         """The bytes to send for a query power restore state."""
         return self.construct_message(bytearray([0x32, 0x3A, 0x3B, 0x0F]))
 
+    def construct_get_time(self) -> bytearray:
+        """Construct a get time command."""
+        return self.construct_message(bytearray([0x11, 0x1A, 0x1B, 0x0F]))
+
+    def is_valid_get_time_response(self, msg: bytes) -> bool:
+        """Check if the response is a valid time response."""
+        return (
+            _message_type_from_start_of_msg(msg) == MSG_TIME
+            and len(msg) == LEDENET_REMOTE_CONFIG_TIME_RESPONSE_LEN
+            and self.is_checksum_correct(msg)
+        )
+
+    def parse_get_time(self, rx: bytes) -> Optional[datetime.datetime]:
+        """Parse a get time command."""
+        if self.is_valid_get_time_response(rx):
+            with contextlib.suppress(Exception):
+                return datetime.datetime(
+                    rx[3] + 2000, rx[4], rx[5], rx[6], rx[7], rx[8]
+                )
+        return None
+
+    def construct_set_time(self, time: Optional[datetime.datetime]) -> bytearray:
+        """Construct a set time command."""
+        dt = time or datetime.datetime.now()
+        return self.construct_message(
+            bytearray(
+                [
+                    0x10,
+                    0x14,
+                    dt.year - 2000,
+                    dt.month,
+                    dt.day,
+                    dt.hour,
+                    dt.minute,
+                    dt.second,
+                    dt.isoweekday(),  # day of week
+                    0x00,
+                    0x0F,
+                ]
+            )
+        )
+
     def construct_power_restore_state_change(
         self, restore_state: PowerRestoreStates
     ) -> bytearray:
@@ -498,13 +563,13 @@ class ProtocolBase:
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request."""
 
     @abstractmethod
@@ -581,6 +646,66 @@ class ProtocolBase:
     ) -> Union[LEDENETOriginalRawState, LEDENETRawState]:
         """Convert raw_state to a namedtuple."""
 
+    @abstractmethod
+    def is_valid_remote_config_response(self, msg: bytes) -> bool:
+        """Check if a remote config response is valid."""
+        return _message_type_from_start_of_msg(
+            msg
+        ) == MSG_REMOTE_CONFIG and self.is_checksum_correct(msg)
+
+    def construct_query_remote_config(self) -> bytearray:
+        """Construct a remote config query"""
+        return self.construct_wrapped_message(bytearray([0x2B, 0x2C, 0x2D]))
+
+    def construct_remote_config(self, remote_config: RemoteConfig) -> bytearray:
+        """Construct an remote config."""
+        # 2a 02 ff ff ff ff ff 00 00 00 00 00 00 00 0f
+        return self.construct_wrapped_message(
+            bytearray(
+                [
+                    0x2A,
+                    remote_config.value,
+                    0xFF,
+                    0xFF,
+                    0xFF,
+                    0xFF,
+                    0xFF,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x0F,
+                ]
+            )
+        )
+
+    def construct_unpair_remotes(self) -> bytearray:
+        """Construct an unpair remotes command."""
+        return self.construct_wrapped_message(
+            bytearray(
+                [
+                    0x2A,
+                    0xFF,
+                    0xFF,
+                    0x01,
+                    0xFF,
+                    0xFF,
+                    0xFF,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0x00,
+                    0xF0,
+                ]
+            )
+        )
+
 
 class ProtocolLEDENETOriginal(ProtocolBase):
     """The original LEDENET protocol with no checksums."""
@@ -623,13 +748,13 @@ class ProtocolLEDENETOriginal(ProtocolBase):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request."""
         # sample message for original LEDENET protocol (w/o checksum at end)
         #  0  1  2  3  4
@@ -640,7 +765,11 @@ class ProtocolLEDENETOriginal(ProtocolBase):
         #  |  |  green
         #  |  red
         #  head
-        return self.construct_message(bytearray([0x56, red, green, blue, 0xAA]))
+        return [
+            self.construct_message(
+                bytearray([0x56, red or 0x00, green or 0x00, blue or 0x00, 0xAA])
+            )
+        ]
 
     def construct_message(self, raw_bytes: bytearray) -> bytearray:
         """Original protocol uses no checksum."""
@@ -661,13 +790,13 @@ class ProtocolLEDENETOriginalCCT(ProtocolLEDENETOriginal):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request."""
         # sample message for original LEDENET protocol (w/o checksum at end)
         #  0  1  2  3  4
@@ -678,7 +807,9 @@ class ProtocolLEDENETOriginalCCT(ProtocolLEDENETOriginal):
         #  |  |  green
         #  |  red
         #  head
-        return self.construct_message(bytearray([0x56, red, green, 0xAA]))
+        return [
+            self.construct_message(bytearray([0x56, red or 0x00, green or 0x00, 0xAA]))
+        ]
 
 
 class ProtocolLEDENET8Byte(ProtocolBase):
@@ -739,13 +870,13 @@ class ProtocolLEDENET8Byte(ProtocolBase):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request."""
         # sample message for 8-byte protocols (w/ checksum at end)
         #  0  1  2  3  4  5  6
@@ -771,19 +902,21 @@ class ProtocolLEDENET8Byte(ProtocolBase):
         # value (f0).
         #
         # For all other rgb and rgbw devices, the value is 00
-        return self.construct_message(
-            bytearray(
-                [
-                    0x31 if persist else 0x41,
-                    red,
-                    green,
-                    blue,
-                    warm_white,
-                    write_mode.value,
-                    0x0F,
-                ]
+        return [
+            self.construct_message(
+                bytearray(
+                    [
+                        0x31 if persist else 0x41,
+                        red or 0x00,
+                        green or 0x00,
+                        blue or 0x00,
+                        warm_white or 0x00,
+                        write_mode.value,
+                        0x0F,
+                    ]
+                )
             )
-        )
+        ]
 
     def construct_message(self, raw_bytes: bytearray) -> bytearray:
         """Calculate checksum of byte array and add to end."""
@@ -1002,13 +1135,13 @@ class ProtocolLEDENET9Byte(ProtocolLEDENET8Byte):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request."""
         # sample message for 9-byte LEDENET protocol (w/ checksum at end)
         #  0  1  2  3  4  5  6  7
@@ -1023,20 +1156,22 @@ class ProtocolLEDENET9Byte(ProtocolLEDENET8Byte):
         #  |  red
         #  persistence (31 for true / 41 for false)
         #
-        return self.construct_message(
-            bytearray(
-                [
-                    0x31 if persist else 0x41,
-                    red,
-                    green,
-                    blue,
-                    warm_white,
-                    cool_white,
-                    write_mode.value,
-                    0x0F,
-                ]
+        return [
+            self.construct_message(
+                bytearray(
+                    [
+                        0x31 if persist else 0x41,
+                        red or 0x00,
+                        green or 0x00,
+                        blue or 0x00,
+                        warm_white or 0x00,
+                        cool_white or 0x00,
+                        write_mode.value,
+                        0x0F,
+                    ]
+                )
             )
-        )
+        ]
 
 
 class ProtocolLEDENET9ByteAutoOn(ProtocolLEDENET9Byte):
@@ -1266,36 +1401,43 @@ class ProtocolLEDENETAddressableA2(ProtocolLEDENETAddressableBase):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request.
 
         white  41 01 ff ff ff 00 00 00 60 ff 00 00 9e
         """
         preset_number = 0x01  # aka fixed color
-        return self.construct_message(
-            bytearray(
-                [
-                    0x41,
-                    preset_number,
-                    red,
-                    green,
-                    blue,
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x60,
-                    0xFF,
-                    0x00,
-                    0x00,
-                ]
+        msgs = []
+        if red is not None or green is not None or blue is not None:
+            msgs.append(
+                self.construct_message(
+                    bytearray(
+                        [
+                            0x41,
+                            preset_number,
+                            red or 0x00,
+                            green or 0x00,
+                            blue or 0x00,
+                            0x00,
+                            0x00,
+                            0x00,
+                            0x60,
+                            0xFF,
+                            0x00,
+                            0x00,
+                        ]
+                    )
+                )
             )
-        )
+        if warm_white is not None:
+            msgs.append(self.construct_message(bytearray([0x47, warm_white or 0x00])))
+        return msgs
 
     def construct_music_mode(
         self,
@@ -1591,13 +1733,13 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request.
 
         b0 [unknown static?] b1 [unknown static?] b2 [unknown static?] b3 [unknown static?] 00 [unknown static?] 01 [unknown static?] 01 [unknown static?] 6a [incrementing sequence number] 00 [unknown static?] 0d [unknown, sometimes 0c] 41 [unknown static?] 02 [preset number] ff [foreground r] 00 [foreground g] 00 [foreground b] 00 [background red] ff [background green] 00 [background blue] 06 [speed or direction?] 00 [unknown static?] 00 [unknown static?] 00 [unknown static?] 47 [speed or direction?] cd [check sum]
@@ -1639,12 +1781,12 @@ class ProtocolLEDENETAddressableA3(ProtocolLEDENETAddressableA2):
         Set Red
         b0b1b2b30001010d0034a0000600010000ff0000ff0002ff00000000ff00030000ff0000ff0004ff00000000ff00050000ff0000ff0006ff00000000ffaf67
         """
-        return self.construct_wrapped_message(
-            super().construct_levels_change(
+        return [
+            self.construct_wrapped_message(msg, inner_pre_constructed=True)
+            for msg in super().construct_levels_change(
                 persist, red, green, blue, warm_white, cool_white, write_mode
-            ),
-            inner_pre_constructed=True,
-        )
+            )
+        ]
 
     def construct_zone_change(
         self,
@@ -1758,38 +1900,42 @@ class ProtocolLEDENETCCT(ProtocolLEDENET9Byte):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request.
 
         b0 b1 b2 b3 00 01 01 52 00 09 35 b1 00 64 00 00 00 03 4d bd - 100% warm
         b0 b1 b2 b3 00 01 01 72 00 09 35 b1 64 64 00 00 00 03 b1 a5 - 100% cool
         b0 b1 b2 b3 00 01 01 9f 00 09 35 b1 64 32 00 00 00 03 7f 6e - 100% cool - dim 50%
         """
+        assert warm_white is not None
+        assert cool_white is not None
         scaled_temp, brightness = white_levels_to_scaled_color_temp(
             warm_white, cool_white
         )
-        return self.construct_wrapped_message(
-            bytearray(
-                [
-                    0x35,
-                    0xB1,
-                    scaled_temp,
-                    # If the brightness goes below the precision the device
-                    # will flip from cold to warm
-                    max(self.MIN_BRIGHTNESS, brightness),
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x03,
-                ]
+        return [
+            self.construct_wrapped_message(
+                bytearray(
+                    [
+                        0x35,
+                        0xB1,
+                        scaled_temp,
+                        # If the brightness goes below the precision the device
+                        # will flip from cold to warm
+                        max(self.MIN_BRIGHTNESS, brightness),
+                        0x00,
+                        0x00,
+                        0x00,
+                        0x03,
+                    ]
+                )
             )
-        )
+        ]
 
 
 class ProtocolLEDENETAddressableChristmas(ProtocolLEDENETAddressableBase):
@@ -1845,16 +1991,15 @@ class ProtocolLEDENETAddressableChristmas(ProtocolLEDENETAddressableBase):
     def construct_levels_change(
         self,
         persist: int,
-        red: int,
-        green: int,
-        blue: int,
-        warm_white: int,
-        cool_white: int,
+        red: Optional[int],
+        green: Optional[int],
+        blue: Optional[int],
+        warm_white: Optional[int],
+        cool_white: Optional[int],
         write_mode: LevelWriteMode,
-    ) -> bytearray:
+    ) -> List[bytearray]:
         """The bytes to send for a level change request.
 
-
         Green 100%:
         b0b1b2b300010180000d3ba100646400000000000000a49d
 
@@ -1902,25 +2047,30 @@ class ProtocolLEDENETAddressableChristmas(ProtocolLEDENETAddressableBase):
         Single - Blue - Brightness 50%
         3b a1 78 64 32 00 00 00 00 00 00 00 ea
         """
+        assert red is not None
+        assert green is not None
+        assert blue is not None
         h, s, v = colorsys.rgb_to_hsv(red / 255, green / 255, blue / 255)
-        return self.construct_wrapped_message(
-            bytearray(
-                [
-                    0x3B,
-                    0xA1,
-                    int(h * 180),
-                    int(s * 100),
-                    int(v * 100),
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x00,
-                    0x00,
-                ]
+        return [
+            self.construct_wrapped_message(
+                bytearray(
+                    [
+                        0x3B,
+                        0xA1,
+                        int(h * 180),
+                        int(s * 100),
+                        int(v * 100),
+                        0x00,
+                        0x00,
+                        0x00,
+                        0x00,
+                        0x00,
+                        0x00,
+                        0x00,
+                    ]
+                )
             )
-        )
+        ]
 
     def construct_zone_change(
         self,
